/*
 GBCartRead - Read RAM only
 Author: Alex from Inside Gadgets (http://www.insidegadgets.com)
 Created: 22/03/2011
 Last Modified: 27/03/2011
 
 Read all 512bytes RAM from MBC2
 
 */
 
int latchPin = 10;
int dataPin = 11;
int clockPin = 12;
int rdPin = A5;
int wrPin = 13;
int mreqPin = A4;

void setup() {
  Serial.begin(57600);
  pinMode(latchPin, OUTPUT);
  pinMode(clockPin, OUTPUT);
  pinMode(dataPin, OUTPUT);
  pinMode(rdPin, OUTPUT);
  pinMode(wrPin, OUTPUT);
  pinMode(mreqPin, OUTPUT);
  for (int l = 2; l <= 9; l++) {
    pinMode(l, INPUT);
  }
}

void loop() {
  
  // Wait for serial input
  while (Serial.available() <= 0) {
    delay(200);
  }
  Serial.flush();
  
  // MBC2 Fix (unknown why this fixes it, maybe has to read ROM before RAM?)
  digitalWrite(wrPin, HIGH); // WR off     
  digitalWrite(latchPin, LOW);
  shiftOut(dataPin, clockPin, MSBFIRST, (0x0134 >> 8));
  shiftOut(dataPin, clockPin, MSBFIRST, (0x0134 & 0xFF));
  digitalWrite(latchPin, HIGH);
  
  Serial.println("START");
  
  // Turn everything off
  digitalWrite(rdPin, HIGH); // RD off
  digitalWrite(wrPin, HIGH); // WR off
  digitalWrite(mreqPin, HIGH); // MREQ off
  
  unsigned int addr = 0; // Now we really need to use unsigned
  
  // Shift out 0x0000
  digitalWrite(latchPin, LOW);
  shiftOut(dataPin, clockPin, MSBFIRST, (addr >> 8));
  shiftOut(dataPin, clockPin, MSBFIRST, (addr & 0xFF));
  digitalWrite(latchPin, HIGH);
  delay(1);
  
  // Set D0-D7 pins as outputs
  for (int l = 2; l <= 9; l++) {
    pinMode(l, OUTPUT);
  }
  
  // Initialise MBC: 0x0A
  digitalWrite(3, HIGH);
  digitalWrite(5, HIGH);
  digitalWrite(wrPin, LOW); // WR on
  digitalWrite(wrPin, HIGH); // WR off

  // Turn outputs off and change back to inputs
  for (int l = 2; l <= 9; l++) {
    digitalWrite(l, LOW);
    pinMode(l, INPUT);
  }

  // Read RAM (512 addresses)
  for (addr = 0xA000; addr <= 0xA1FF; addr++) {  
    digitalWrite(latchPin, LOW);
    shiftOut(dataPin, clockPin, MSBFIRST, (addr >> 8));
    shiftOut(dataPin, clockPin, MSBFIRST, (addr & 0xFF));
    digitalWrite(latchPin, HIGH);
    delayMicroseconds(50);
    
    // Tell MBC to process our RAM request
    digitalWrite(mreqPin, LOW); //MREQ on
    digitalWrite(rdPin, LOW); // RD on

    byte bval = 0xF0; // Upper 4 bits always high (11110000)
    for (int z = 9; z >= 2; z--) { // Read only 4 lower bits
      if (digitalRead(z) == HIGH) {
        bitWrite(bval, (z-2), HIGH);
      }
    }
    
    // Done reading this part of RAM
    digitalWrite(mreqPin, HIGH); //MREQ off
    digitalWrite(rdPin, HIGH); // RD off
    
    Serial.println(bval, DEC);
  }
  Serial.println("END");
  
  // Disable RAM
  addr = 0;
  digitalWrite(latchPin, LOW);
  shiftOut(dataPin, clockPin, MSBFIRST, (addr >> 8));
  shiftOut(dataPin, clockPin, MSBFIRST, (addr & 0xFF));
  digitalWrite(latchPin, HIGH);
  delay(1);
  
  // Set D0-D7 pins as outputs
  for (int l = 2; l <= 9; l++) {
    pinMode(l, OUTPUT);
    digitalWrite(l, LOW);
  }
  
  // Tell MBC to process our request
  digitalWrite(wrPin, LOW); // WR 0
  digitalWrite(wrPin, HIGH); // WR 1

  // Set pins back to inputs
  for (int l = 2; l <= 9; l++) {
    pinMode(l, INPUT);
  }
}